#!/usr/bin/perl

   # this subroutine changes a number to the w3c timeformat
   # the number is the time in seconds passed after 1 jan 1970
   # same as gmtime from perl does.
   # But this sub returns the W3C time format.
   
   # (c) RJHM van den Bergh, Comweb Netherlands.
   # Please report bugs and sugestions to sales2@comweb.nl
   # version 1.0 betta
   
   # No liability accepted
   # Free to use and distribute
   # please keep my name in the source code       

   use strict;
   use warnings;
   use POSIX;
   
# For testing
print "\n";
print "Testing w3ctime function\n";
print "See https://www.w3.org/TR/NOTE-datetime \n";
print "\n";

print "w3ctime(0)= ".&w3ctime("0")."\n";
print "gmtime(0) = ".gmtime(0)."\n";
print "\n";

print "w3ctime(86400)= ".&w3ctime("86400")."\n";
print "gmtime(86400) = ".gmtime(86400)."\n";
print "\n";

print "w3ctime(1549658691)= ".&w3ctime("1549658691")."\n";
print "gmtime(1549658691) = ".gmtime(1549658691)."\n";
print "\n";

sub w3ctime()
{
    my $time_passed_since_epoch=shift;  #  time in seconds since 1 jan 1970 00:00:00
    # initialize some variables
    my $year=1970;
    my $isLeapYear=0;
    my $leapYearInSeconds=366*24*60*60;
    my $yearInSeconds    =365*24*60*60;
    my $start_counter=0;
    
    
    while( $start_counter<=$time_passed_since_epoch)
    {
      #determine if it is a leap year      
      if (($year % 4)==0   ) { $isLeapYear=1; } else { $isLeapYear=0; }
      if (($year % 100)==0 ) { $isLeapYear=0; } # exception
      if (($year % 400)==0 ) { $isLeapYear=1; } # exception      
      
      # add seconds of that year to the start counter
      if( $isLeapYear ) 
      {
         $start_counter+=$leapYearInSeconds; 
      } else
      {
         $start_counter+=$yearInSeconds;
      }      
      $year++;      
    }
    # $tart passed $time_passed
    $year--;
    if( $isLeapYear ) 
    {
      $start_counter-=$leapYearInSeconds; 
    } else
    {
      $start_counter-=$yearInSeconds;
    }
    
    my $month=0;
    # can add jan 31 days?
    if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
    {
       $start_counter+=31*24*60*60;  
       $month++;

       # can add februari?
       my $februariDays=28;
       if ($isLeapYear) { $februariDays=29;}
       if ( ($start_counter+$februariDays*24*60*60)<=$time_passed_since_epoch )
       {
          $start_counter+=$februariDays*24*60*60;  
          $month++;
       
          # can add march?
          if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
          {
             $start_counter+=31*24*60*60;  
             $month++;
       
             # can add april?
             if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
             {
                $start_counter+=30*24*60*60;  
                $month++;

                # can add may?
                if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                {
                   $start_counter+=31*24*60*60;  
                   $month++;
                
                   # can add june?
                   if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
                   {
                      $start_counter+=30*24*60*60;  
                      $month++;

                      # can add july?
                      if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                      {
                         $start_counter+=31*24*60*60;  
                         $month++;

                         # can add august?
                         if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                         {
                            $start_counter+=31*24*60*60;  
                            $month++;

                            # can add september?
                            if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
                            {
                               $start_counter+=30*24*60*60;  
                               $month++;

                               # can add oktober?
                               if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                               {
                                  $start_counter+=31*24*60*60;  
                                  $month++;
                               
                                  # can add november?
                                  if ( ($start_counter+30*24*60*60)<=$time_passed_since_epoch )
                                  {
                                     $start_counter+=30*24*60*60;  
                                     $month++;

                                     # can add december?
                                     if ( ($start_counter+31*24*60*60)<=$time_passed_since_epoch )
                                     {
                                        $start_counter+=31*24*60*60;  
                                        $month++;

                                        # if it comes here then something not correct
                                        # should not be possible to add full decembre month
                                        # because then it is next year
                                        die "Oops it shouldn't get here";
                                                                                                                     
                                     }
                                  }
                               }
                            }
                         }
                      }
                   }
                }
             }
          }
       }
    }
    
    # calculate days in the month lowest day = 0
    my $days=floor(($time_passed_since_epoch-$start_counter)/(24*60*60));
    $start_counter+=($days*24*60*60);
    if (($days+1)>31) { die "Oops days can't be bigger than 31.";}
    
    #calculate hours lowest = 0
    my $hours=floor(($time_passed_since_epoch-$start_counter)/(60*60));
    $start_counter+=($hours*60*60);
    if ($hours>=24) { die "Oops hours must be less than 24.";}
    
    # calculate minutes lowest = 0
    my $minutes=floor(($time_passed_since_epoch-$start_counter)/(60));
    $start_counter+=($minutes*60);
    if ($minutes>=60) { die "Oops minutes must be less than 60.";}

    my $seconds=floor($time_passed_since_epoch-$start_counter);
    if ($seconds>=60) { die "Oops seconds must be less than 60.";}
    
    # putting everything in the right format
    $year  = sprintf("%04d", $year); 
    $month = sprintf("%02d", $month+1);
    $days  = sprintf("%02d", $days+1); 
    $hours   = sprintf("%02d", $hours);  
    $minutes = sprintf("%02d", $minutes); 
    $seconds = sprintf("%02d", $seconds);
    
    my $w3cTimeString="$year-".($month)."-".($days)."T".$hours.":".$minutes.":".$seconds."Z";
    return $w3cTimeString; 
}
